@@ -852,8 +852,13 @@ static void set_params(void)
 		}
 	}
 	rate = hwparams.rate;
-	if (buffer_time == 0 && buffer_frames == 0)
-		buffer_time = 500000;
+	if (buffer_time == 0 && buffer_frames == 0) {
+		err = snd_pcm_hw_params_get_buffer_time_max(params,
+							    &buffer_time, 0);
+		assert(err >= 0);
+		if (buffer_time > 500000)
+			buffer_time = 500000;
+	}
 	if (period_time == 0 && period_frames == 0) {
 		if (buffer_time > 0)
 			period_time = buffer_time / 4;
@@ -1922,7 +1927,10 @@ static void capture(char *name)
 	pbrec_count = calc_count();
 	/* WAVE-file should be even (I'm not sure), but wasting one byte
 	   isn't a problem (this can only be in 8 bit mono) */
-	pbrec_count += pbrec_count % 2;
+	if (pbrec_count < LLONG_MAX)
+		pbrec_count += pbrec_count % 2;
+	else
+		pbrec_count -= pbrec_count % 2;
 	if (pbrec_count == 0)
 		pbrec_count -= 2;
 	if (fmt_rec_table[file_type].start)
